@@ -152,6 +152,7 @@ struct userdata {
 struct loopback_msg {
     pa_msgobject parent;
     struct userdata *userdata;
+    bool dead;
 };
 
 PA_DEFINE_PRIVATE_CLASS(loopback_msg, pa_msgobject);
@@ -206,6 +207,9 @@ static void teardown(struct userdata *u) {
     u->adjust_time = 0;
     enable_adjust_timer(u, false);
 
+    if (u->msg)
+        u->msg->dead = true;
+
     /* Handling the asyncmsgq between the source output and the sink input
      * requires some care. When the source output is unlinked, nothing needs
      * to be done for the asyncmsgq, because the source output is the sending
@@ -1016,6 +1020,14 @@ static void set_sink_input_latency(struct userdata *u, pa_sink *sink) {
     if (u->min_source_latency > requested_latency) {
         latency = PA_MAX(u->latency, u->minimum_latency);
         requested_latency = (latency - u->min_source_latency) / 2;
+        /* In the case of a fixed alsa source, u->minimum_latency is calculated from
+         * the default fragment size while u->min_source_latency is the reported minimum
+         * of the source latency (nr_of_fragments * fragment_size). This can lead to a
+         * situation where u->minimum_latency < u->min_source_latency. We only fall
+         * back to use the fragment size instead of min_source_latency if the calculation
+         * above does not deliver a usable result. */
+        if (u->fixed_alsa_source && u->min_source_latency >= latency)
+            requested_latency = (latency - u->core->default_fragment_size_msec * PA_USEC_PER_MSEC) / 2;
     }
 
     latency = PA_CLAMP(requested_latency , u->min_sink_latency, u->max_sink_latency);
@@ -1219,6 +1231,12 @@ static int loopback_process_msg_cb(pa_msgobject *o, int code, void *userdata, in
     pa_assert_ctl_context();
 
     msg = LOOPBACK_MSG(o);
+
+    /* If messages are processed after a module unload request, they
+     * must be ignored. */
+    if (msg->dead)
+        return 0;
+
     pa_assert_se(u = msg->userdata);
 
     switch (code) {
@@ -1605,6 +1623,7 @@ int pa__init(pa_module *m) {
     u->msg = pa_msgobject_new(loopback_msg);
     u->msg->parent.process_msg = loopback_process_msg_cb;
     u->msg->userdata = u;
+    u->msg->dead = false;
 
     /* The output thread is not yet running, set effective_source_latency directly */
     update_effective_source_latency(u, u->source_output->source, NULL);
@@ -1648,5 +1667,8 @@ void pa__done(pa_module*m) {
     if (u->asyncmsgq)
         pa_asyncmsgq_unref(u->asyncmsgq);
 
+    if (u->msg)
+        loopback_msg_unref(u->msg);
+
     pa_xfree(u);
 }
