@@ -111,7 +111,8 @@ static void remap_mono_to_stereo_s16ne_mmx(pa_remap_t *m, int16_t *dst, const in
     );
 }
 
-static void remap_mono_to_stereo_float32ne_mmx(pa_remap_t *m, float *dst, const float *src, unsigned n) {
+/* Works for both S32NE and FLOAT32NE */
+static void remap_mono_to_stereo_any32ne_mmx(pa_remap_t *m, float *dst, const float *src, unsigned n) {
     pa_reg_x86 temp, temp2;
 
     __asm__ __volatile__ (
@@ -135,7 +136,8 @@ static void init_remap_mmx(pa_remap_t *m) {
 
         pa_log_info("Using MMX mono to stereo remapping");
         pa_set_remap_func(m, (pa_do_remap_func_t) remap_mono_to_stereo_s16ne_mmx,
-            (pa_do_remap_func_t) remap_mono_to_stereo_float32ne_mmx);
+            (pa_do_remap_func_t) remap_mono_to_stereo_any32ne_mmx,
+            (pa_do_remap_func_t) remap_mono_to_stereo_any32ne_mmx);
     }
 }
 #endif /* defined (__i386__) || defined (__amd64__) */
