@@ -114,7 +114,7 @@ static void run_mix_test(
         for (i = 0; i < nsamples; i++) {
             if (samples[i] != samples_ref[i]) {
                 pa_log_debug("Correctness test failed: align=%d, channels=%d", align, channels);
-                pa_log_debug("%d: %hd != %04hd (%hd + %hd)\n",
+                pa_log_debug("%d: %hd != %04hd (%hd + %hd)",
                     i,
                     samples[i], samples_ref[i],
                     samples0[i], samples1[i]);
